
class RenderingExample(object):
    def __init__(self,level=0):
        self.points = [
            [  1.0 ,  0.0 ,  0.0 ],[ -1.0 ,  0.0 ,  0.0 ],
            [  0.0 ,  1.0 ,  0.0 ],[  0.0 , -1.0 ,  0.0 ],
            [  0.0 ,  0.0 ,  1.0 ],[  0.0 ,  0.0 , -1.0 ]
        ]
        self.triangles = [
            [2,4,0],[2,0,5],[2,5,1],[2,1,4],[3,0,4],[3,5,0],[3,1,5],[3,4,1]
        ]
        self.pitch = 0.0
        self.yaw = 0.0
        self.level=level
        self.context=None
        
    def subdivide(self,point0,point1,point2,level):
        gl = zort.plugins.opengl.gl
        midpoint = [ [0,0,0],[0,0,0],[0,0,0]]
        #/* Don't subdivide any further; just draw the triangle*/
        if level==0:
            gl.Color3f(*point0)
            gl.Vertex3f(*point0)
            gl.Color3f(*point1)
            gl.Vertex3f(*point1)
            gl.Color3f(*point2)
            gl.Vertex3f(*point2)
        else:
            for coord in range(0,3):
                midpoint[0][coord] = (point0[coord] + point1[coord])*0.5
                midpoint[1][coord] = (point1[coord] + point2[coord])*0.5
                midpoint[2][coord] = (point2[coord] + point0[coord])*0.5
            # /* Subdivide each triangle into three more */    /*     .      */
            level=level-1
            self.subdivide(point0,midpoint[0],midpoint[2],level)
            self.subdivide(point1,midpoint[1],midpoint[0],level)
            self.subdivide(point2,midpoint[2],midpoint[1],level)
            
    def draw(self,window):
        window.lock()
        try:
            if not self.context:
                self.context = zort.plugins.opengl.OpenGL.createContext(window)
            self.context.makeCurrent(window)
            #
            gl = zort.plugins.opengl.gl
            glu = zort.plugins.opengl.glu
            # init
            gl.Viewport(0, 0, window.width(), window.height())
            gl.MatrixMode(gl.GL_PROJECTION)
            gl.LoadIdentity()
            glu.Perspective(60.0, float(window.width())/float(window.height()), 0.1, 100.0)
            gl.MatrixMode(gl.GL_MODELVIEW)
            gl.ClearColor(0.5, 0.5 ,0.5, 0.0)
            gl.Enable(gl.GL_DEPTH_TEST)
            gl.Disable(gl.GL_CULL_FACE)
            # end init
            #/* Clear the color plane and the z-buffer */
            gl.Clear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
            gl.LoadIdentity()
            #    /* Move the object 2 units away from the camera */
            gl.Translatef(0.0, 0.0, -2.0)
            #    /* Rotate the object */
            gl.Rotatef(self.pitch,1.0,0.0,0.0)
            gl.Rotatef(self.yaw,0.0,1.0,0.0)
            #    /* Draw the triangles which make up the object */
            gl.Begin(gl.GL_TRIANGLES)
            #
            points = self.points
            triangles = self.triangles
            for i in range(0,8):
                self.subdivide(points[triangles[i][0]],points[triangles[i][1]],points[triangles[i][2]],self.level)
            #    
            gl.End()
            #
            self.context.swapBuffers(window)
        finally:
            window.unlock()
        
    def __call__(self,window):
        self.pitch += 0.1
        self.yaw += 0.1
        self.draw(window)

class ExampleIkosaeder(object):
	def __init__(self):
		self.context = None
		self.spinned_x = 0.0
		self.spinned_y = 0.0
		# generate triangles
		import math
		# goldene schnitt
		ratio = ( 1.0 + math.sqrt(5.0) ) / 2.0
		a = 0.7
		b = 0.7/ratio
		# a > b
		points = [
			# rectangle 1
			[ (b,a,0),(-b,a,0),(-b,-a,0),(b,-a,0) ],
			# rectangle 2
			[ (0,b,a),(0,-b,a),(0,-b,-a),(0,b,-a) ],
			# rectangle 3
			[ (a,0,b),(a,0,-b),(-a,0,-b),(-a,0,b) ],
		]
		
		# generate triangles
		self.triangles = []
		
		# rect 1 pos ratio
		self.triangles += [ [ points[0][0], points[0][1], points[1][0]]  ]
		self.triangles += [ [ points[0][0], points[0][1], points[1][3]]  ]
		# rect 1 neg ratio
		self.triangles += [ [ points[0][2], points[0][3], points[1][1]]  ]
		self.triangles += [ [ points[0][2], points[0][3], points[1][2]]  ]
		# rect 2 pos ratio
		self.triangles += [ [ points[1][0], points[1][1], points[2][0]]  ]
		self.triangles += [ [ points[1][0], points[1][1], points[2][3]]  ]
		# rect 2 neg ratio
		self.triangles += [ [ points[1][2], points[1][3], points[2][1]]  ]
		self.triangles += [ [ points[1][2], points[1][3], points[2][2]]  ]
		# rect 3 pos ratio
		self.triangles += [ [ points[2][0], points[2][1], points[0][0]]  ]
		self.triangles += [ [ points[2][0], points[2][1], points[0][3]]  ]
		# rect 3 neg ratio
		self.triangles += [ [ points[2][2], points[2][3], points[0][1]]  ]
		self.triangles += [ [ points[2][2], points[2][3], points[0][2]]  ]
		# inter rectangle :) - trial and error
		self.triangles += [ [ points[0][0], points[1][0], points[2][0]]  ]
		self.triangles += [ [ points[0][2], points[1][2], points[2][2]]  ]
		self.triangles += [ [ points[0][1], points[1][3], points[2][2]]  ]
		self.triangles += [ [ points[0][3], points[1][2], points[2][1]]  ]
		self.triangles += [ [ points[0][3], points[1][1], points[2][0]]  ]
		self.triangles += [ [ points[0][0], points[1][3], points[2][1]]  ]
		self.triangles += [ [ points[0][2], points[1][1], points[2][3]]  ]
		self.triangles += [ [ points[0][1], points[1][0], points[2][3]]  ]
		
		#self.triangles += [ [ points[0][2], points[1][1], points[2][3]]  ]
		#self.triangles += [ [ points[0][2], points[1][1], points[2][3]]  ]
		
		# set up colors
		self.colors = [
			(0,0,1),(0,1,0),(1,0,0),
			(1,1,0),(1,0,1),(0,1,1),
			(1,0.5,0.5),(0.5,1,0.5),(0.5,0.5,1),
			(1,1,0.5),(1,1,0.5),(1,0.5,1),
			(0.5,0.5,0.5),
			(0.3,0.8,0.3),(0.8,0.3,0.3),(0.3,0.3,0.8),
			(0.8,0.8,0.3),(0.8,0.3,0.8),(0.3,0.8,0.8),
			(0.8,0.8,0.8),(0.3,0.3,0.3),
			]
		self.color_index = 0
	def __call__(self,window):
		window.lock()
		try:
			if not self.context:
				self.context = zort.plugins.opengl.OpenGL.createContext(window)
			self.context.makeCurrent(window)
			self.spinned_x += 0.4
			self.spinned_y += 0.6
			gl = zort.plugins.opengl.gl
			glu = zort.plugins.opengl.glu
			# init
			#print window.width(), window.height()
			gl.Viewport(0, 0, window.width(), window.height())
			gl.MatrixMode(gl.GL_PROJECTION)
			gl.LoadIdentity()
			glu.Perspective(60.0, float(window.width())/float(window.height()), 0.1, 100.0)
			gl.MatrixMode(gl.GL_MODELVIEW)
			gl.ClearColor(0.0, 0.0 ,0.0, 0.0)
			gl.Enable(gl.GL_DEPTH_TEST)
			gl.Disable(gl.GL_CULL_FACE)
			# end init
			gl.LoadIdentity()
			gl.Translatef(0.0, 0.0, -2.0)
			gl.Rotatef(self.spinned_y,0.0,1.0,0.0)
			gl.Rotatef(self.spinned_x,1.0,0.0,0.0)
			gl.Clear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
			gl.Begin(gl.GL_TRIANGLES)
			self.color_index = len(self.colors)
			for triangle in self.triangles:
				self.color_index+=1
				if self.color_index>=len(self.colors):
					self.color_index=0
				for point in triangle:
					gl.Color3f(*self.colors[self.color_index])
					gl.Vertex3f(point[0],point[1],point[2])
			gl.End()
			self.context.swapBuffers(window)
		finally:
			window.unlock()

def create_example_window(level = 0):
    window = zort.plugins.opengl.OpenGL.createWindow(400,400,"example_window level="+str(level))
    draw_function = zort.globals["example_draw_class"](level)
    timer = zort.plugins.timing.Timing.timer(60)
    event = zort.plugins.timing.Timing.createEvent()
    timer.event = event
    event.connect(lambda : draw_function(window) )
    zort.globals["example_timers"] += [timer]
    zort.globals["example_events"] += [event]
    zort.globals["example_windows"] += [window]

class FPSCounter(object):
    def __init__(self,description,interval=5,timer=None):
        self.last_time = zort.plugins.timing.Timing.time()
        self.call_count = 0
        self.interval = interval
        self.description = description
        self.timer = timer
    def __call__(self):
        self.call_count += 1
        now = zort.plugins.timing.Timing.time()
        if (now - self.last_time)>self.interval :
            print "FPS", self.description, ":", (self.call_count/(now-self.last_time))
            if self.timer:
                print "Usage", self.description, ":", self.timer.usage()
            self.last_time=now
            self.call_count=0

class TimingTest(object):
	def __init__(self,interval=0.001):
		self.interval = interval
	def test(self):
		while True:
			start = zort.plugins.timing.Timing.time()
			zort.plugins.timing.Timing.sleep(self.interval)
			stop = zort.plugins.timing.Timing.time()
			print "sleep called with",self.interval,"seconds. difftime=",(stop-start)
			self.interval *= 2
			if self.interval > 0.5:
				self.interval=0.001

zort.globals["example_timers"] = []
zort.globals["example_events"] = []
zort.globals["example_windows"] = []

zort.globals["example_fps_counter_class"] = FPSCounter
zort.globals["example_timing_test_class"] = TimingTest

zort.globals["example_draw"] = RenderingExample()
zort.globals["example_draw_class"] = RenderingExample
zort.globals["example_draw2_class"] = ExampleIkosaeder

zort.globals["example_create_window"] = create_example_window

